﻿using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.Wpf;

namespace GettingStartedSample
{
    public partial class MainWindow : Window
    {
        private LayerOverlay layerOverlay;
        private InMemoryFeatureLayer highlightLayer;
        private string appDataPath = "../../AppData";
        private PointShape worldLocation = new PointShape(0, 0);
        private Collection<Feature> selectedFeatures = new Collection<Feature>();

        public MainWindow()
        {
            InitializeComponent();
        }

        private void map_Loaded(object sender, RoutedEventArgs e)
        {
            map.MapUnit = GeographyUnit.DecimalDegree;
            map.CurrentExtent = new RectangleShape(-133.2515625, 89.2484375, 126.9046875, -88.290625);
            map.MapTools.MouseCoordinate.IsEnabled = true;

            map.TrackOverlay.TrackShapeLayer.WrappingMode = WrappingMode.WrapDateline;
            map.TrackOverlay.TrackShapeLayer.WrappingExtent = new RectangleShape(-180, 90, 180, -90);
            map.EditOverlay.EditShapesLayer.WrappingMode = WrappingMode.WrapDateline;
            map.EditOverlay.EditShapesLayer.WrappingExtent = new RectangleShape(-180, 90, 180, -90);

            WorldStreetsAndImageryOverlay worldOverlay = new WorldStreetsAndImageryOverlay();
            worldOverlay.WrappingMode = WrappingMode.WrapDateline;
            map.Overlays.Add(worldOverlay);

            layerOverlay = new LayerOverlay();
            layerOverlay.TileType = TileType.SingleTile;
            layerOverlay.WrappingMode = WrappingMode.WrapDateline;
            layerOverlay.WrappingExtent = new RectangleShape(-180, 90, 180, -90);
            highlightLayer = new InMemoryFeatureLayer();
            highlightLayer.WrappingMode = WrappingMode.WrapDateline;
            highlightLayer.WrappingExtent = new RectangleShape(-180, 90, 180, -90);
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoBrushes.Red, 1));
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoBrushes.Red, 1));
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(new GeoFont(), 0, GeoBrushes.Red);
            highlightLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            PolygonShape poly2 = new PolygonShape("POLYGON ((180 48.5, 180 25, 155.31734 25, 155.36723 25.05004, 155.74595 25.42996, 155.92059 25.60514, 155.98318 25.66794, 156.00022 25.68503, 156.09598 25.78109, 156.14967 25.83495, 156.18561 25.871, 156.23202 25.91756, 156.3142 26, 160 26, 160 46, 155 46, 155 50, 165.66667 50, 165.66667 48.5, 178.91874 48.5, 180 48.5))");
            CustomFeature tempFeature = new CustomFeature(poly2);
            tempFeature.WrapGeomotry();
            highlightLayer.InternalFeatures.Add(tempFeature);

            layerOverlay.Layers.Add(highlightLayer);
            map.Overlays.Add(layerOverlay);

            map.Refresh();
        }

        private void button_Click(object sender, RoutedEventArgs e)
        {
            Button button = sender as Button;
            if (button != null)
            {
                switch (button.Name)
                {
                    case "btnTrackNormal":
                        map.TrackOverlay.TrackMode = TrackMode.None;
                        break;

                    case "btnTrackPoint":
                        map.TrackOverlay.TrackMode = TrackMode.Point;
                        break;

                    case "btnTrackLine":
                        map.TrackOverlay.TrackMode = TrackMode.Line;
                        break;

                    case "btnTrackRectangle":
                        map.TrackOverlay.TrackMode = TrackMode.Rectangle;
                        break;

                    case "btnTrackSquare":
                        map.TrackOverlay.TrackMode = TrackMode.Square;
                        break;

                    case "btnTrackPolygon":
                        map.TrackOverlay.TrackMode = TrackMode.Polygon;
                        break;

                    case "btnTrackCircle":
                        map.TrackOverlay.TrackMode = TrackMode.Circle;
                        break;

                    case "btnTrackEllipse":
                        map.TrackOverlay.TrackMode = TrackMode.Ellipse;
                        break;

                    case "btnTrackEdit":
                        map.TrackOverlay.TrackMode = TrackMode.None;
                        foreach (Feature feature in map.TrackOverlay.TrackShapeLayer.InternalFeatures)
                        {
                            map.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature);
                        }

                        map.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();
                        map.EditOverlay.CalculateAllControlPoints();
                        map.Refresh(new Overlay[] { map.EditOverlay, map.TrackOverlay });
                        break;

                    case "btnTrackSave":
                        foreach (Feature feature in map.EditOverlay.EditShapesLayer.InternalFeatures)
                            map.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(feature);

                        map.EditOverlay.EditShapesLayer.InternalFeatures.Clear();
                        map.EditOverlay.CalculateAllControlPoints();
                        map.Refresh(new Overlay[] { map.EditOverlay, map.TrackOverlay });

                        string tempFolder = "Temp";
                        if (!Directory.Exists(tempFolder)) Directory.CreateDirectory(tempFolder);
                        foreach (string file in Directory.GetFiles(tempFolder)) File.Delete(file);

                        foreach (Feature feature in map.TrackOverlay.TrackShapeLayer.InternalFeatures)
                        {
                            string tempFileName = Path.Combine(tempFolder, feature.Id + ".txt");
                            CustomFeature tempFeature = new CustomFeature(feature);
                            tempFeature.UnwrapGeomotry();
                            File.WriteAllText(tempFileName, tempFeature.GetWellKnownText());
                        }
                        break;

                    case "btnTrackDelete":
                        int lastIndex = map.EditOverlay.EditShapesLayer.InternalFeatures.Count - 1;

                        if (lastIndex >= 0)
                        {
                            map.EditOverlay.EditShapesLayer.InternalFeatures.RemoveAt(lastIndex);
                            map.EditOverlay.CalculateAllControlPoints();
                        }

                        map.Refresh(map.EditOverlay);
                        break;

                    default:
                        map.TrackOverlay.TrackMode = TrackMode.None;
                        break;
                }
            }
        }
    }
}
